#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `openbases` package."""


import unittest
import os
import sys

from openbases.main import Client
from openbases.logger import bot
test_dir = os.path.abspath(os.path.dirname(__file__))

class TestOpenBasesValidator(unittest.TestCase):
    """Tests for `openbases` package."""

    def setUp(self):
        self.cli = Client
        self.md = '%s/paper/paper.md' % test_dir
        self.validator = self.cli.PaperValidator(self.md)

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_validation(self):
        """Test that the markdown is loaded fully"""
        repo = os.path.join(test_dir, '../../')
        bot.info("repo: %s" % repo)
        os.environ['OPENBASESENV_REPO_BASE'] = repo
        self.validator.validate_criteria()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot

import json
import sys
import os
import re

from .command import ( init_command, run_command )
from .flags import parse_verbosity
from .logger import ( println,  init_level )
from .generate import RobotNamer

class Client:

    def __str__(self):
        return "[openbases-python]"

    def __repr__(self):
        return self.__str__()

    def version(self):
        '''return the version of openbases python
        '''
        from openbases.version import __version__
        return __version__



# Commands
Client._init_command = init_command
Client._run_command = run_command

# Flags and Logger
Client._parse_verbosity = parse_verbosity
Client._println = println
Client._init_level = init_level
Client.RobotNamer = RobotNamer()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
import os

def main(args, options=None):

    from openbases.main import Client

    params = None
    repo = args.repo
    if repo is None:
        repo = os.getcwd()
    repo = os.path.abspath(repo)

    if os.path.exists(repo):
        bot.info("repo: %s" % repo)
        os.environ['OPENBASESENV_REPO_BASE'] = repo
        os.putenv('OPENBASESENV_REPO_BASE', repo)
    else:
        bot.warning('%s does not exist.' % repo)

    # Ensure that paper exists
    paper = args.infile
    if not paper:
        bot.exit('You must specify a paper with --infile')
    if not os.path.exists(paper):
        bot.exit('%s does not exist.' % paper)
    paper = os.path.abspath(paper)     

    if args.basic is True:
        validator = Client.BasicValidator(infile=paper)
    else:
        validator = Client.PaperValidator(infile=paper)
    validator.validate_criteria(criteria=args.criteria, 
                                params=params)



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import os
import re

import json
from openbases.logger import bot
import subprocess
import sys


################################################################################
# Local commands and requests
################################################################################


def check_install(software=None, quiet=True):
    '''check_install will attempt to run some command, and
       return True if installed. The command line utils will not run 
       without this check.
    '''
    if software == None:
        bot.error("Please enter the name of the software!")

    cmd = [software, '--version']
    found = False

    try:
        version = run_command(cmd, quiet=True)
    except: # FileNotFoundError
        return found

    if version is not None:
        if version['return_code'] == 0:
            found = True

        if quiet is False:
            version = version['message']
            bot.info("Found %s version %s" % (software.upper(), version))

    return found


def get_installdir():
    '''get_installdir returns the installation directory of the application
    '''
    return os.path.abspath(os.path.join('..', os.path.dirname(__file__)))


def stream_command(cmd, no_newline_regexp="Progess", sudo=False):
    '''stream a command (yield) back to the user, as each line is available.
       :: note
           # Example usage:
           results = []
           for line in stream_command(cmd):
               print(line, end="")
               results.append(line)
       Parameters
       ==========
       cmd: the command to send, should be a list for subprocess
       no_newline_regexp: the regular expression to determine skipping a
       newline. Defaults to finding Progress
    '''
    if sudo is True:
        cmd = ['sudo'] + cmd

    process = subprocess.Popen(cmd, 
                               stdout = subprocess.PIPE, 
                               universal_newlines = True)
    for line in iter(process.stdout.readline, ""):
        if not re.search(no_newline_regexp, line):
            yield line
    process.stdout.close()
    return_code = process.wait()
    if return_code:
        raise subprocess.CalledProcessError(return_code, cmd)


def run_command(cmd, 
                sudo=False,
                capture=True,
                no_newline_regexp="Progess",
                quiet=False):

    '''run_command uses subprocess to send a command to the terminal. If
       capture is True, we use the parent stdout, so the progress bar (and
       other commands of interest) are piped to the user. This means we 
       don't return the output to parse.
       Parameters
       ==========
       cmd: the command to send, should be a list for subprocess
       sudo: if needed, add to start of command
       no_newline_regexp: the regular expression to determine skipping a
                          newline. Defaults to finding Progress
       capture: if True, don't set stdout and have it go to console. This
       option can print a progress bar, but won't return the lines
       as output.
    '''

    if sudo is True:
        cmd = ['sudo'] + cmd

    stdout = None
    if capture is True:
        stdout = subprocess.PIPE

    # Use the parent stdout and stderr
    process = subprocess.Popen(cmd, 
                               stderr = subprocess.PIPE, 
                               stdout = stdout)
    lines = ()
    found_match = False

    for line in process.communicate():
        if line:
            if isinstance(line, bytes):
                line = line.decode('utf-8')
            lines = lines + (line,)
            if re.search(no_newline_regexp, line) and found_match is True:
                if quiet is False:
                    sys.stdout.write(line)
                found_match = True
            else:
                if quiet is False:
                    sys.stdout.write(line)
                    print(line.rstrip())
                found_match = False

    output = {'message': lines,
              'return_code': process.returncode }

    return output



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import sys

def main(args, options, parser):

    lookup = { 'ipython': ipython,
               'python': python,
               'bpython': bpython }

    shells = ['ipython', 'python', 'bpython']

    for shell in shells:
        try:
            return lookup[shell]
        except ImportError:
            pass
    

def ipython(image):
    import openbases
    from IPython import embed
    embed()

def bpython(image):
    import bpython
    import openbases
    bpython.embed(locals_={'openbases': openbases})

def python(image):
    import code
    import openbases
    code.interact(local={"openbases":openbases})



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# This code was originally written *by same author* for openschemas and shared
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

# These are validation functions referenced in the default (and other)
# criteria.yml files (e.g., paper.yml)

from openbases.logger import bot
from random import choice

def missing(spec):
    '''this function is fired if the user forgets to define a "function" field
       in the check, which is technically the only required one :) It will
       always return False.
    '''
    messages = ['Ruhroh, you forgot to define the function here!',
                'Resistance is futile, and so is a check without a function.',
                'Forgetting something? Like, the function for this check?',
                'The scarecrow (today) sings "If I only had a function..."']

    message = choice(messages)
    print(message)
    return False


def dummy(spec, passing=True):
    '''dummy can be used for testing, it returns the status given as an argument

       Parameters
       ==========
       spec: the input spec, in json format (dict)
       passing: boolean to return True or False (default is True)
    '''
    msg = "not True"
    if passing:
        msg = "True"

    messages = ['Roses are red, violets are blue, here is a test, it is %s' % msg,
                "If I were a rich man, well then I wouldn't be a dinosaur.",
                "Sweet dreams are made of cheese, who am I to diss a brie?"]

    message = choice(messages)
    print(message)
    return passing



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot

import os
import sys

here = os.path.abspath(os.path.dirname(__file__))

      
################################################################################
# Supporting (shared) functions ################################################
################################################################################

def update_environment(params=None, prefix='OPENBASESENV_'):
    '''update environment will put any environment variables found in the 
       params (dictionary) into the environment, expected to be found by
       the various functions. The variables must start with OPENBASESENV_
    '''
    if params is not None:
        for key,value in params.items():
            if key.startswith(prefix):
                if value:
                    bot.debug('Exporting %s=%s' %(key, value))
                    os.putenv(key, value)
                    os.environ[key] = value
            else:
                bot.warning('Skipping parameter %s for environment' % key)


def validate_loads(infile):
    '''determine if a file can load without error. If yes, return manager.
       If not, return None.
           
       Parameters
       ==========
       infile: the input file to attempt loading with the YamlManager,
       can be yaml or front end matter in html.
    '''
    from openbases.utils.managers import YamlManager

    manager = YamlManager(infile)
    try:
        manager.load()
        return manager
    except:
        bot.warning('Load of %s not successfully, using default' % infile)


def load_criteria(self, criteria=None):
    '''load a criteria.yml file. If not specified, load (or reload)
       default provided by package.

       Parameters
       ==========
       criteria: a yml specification for criteria. If not provided, use
        default at criteria/specification.yml. If you need help creating
       a new criteria (that might be added to defaults) please open an issue
    '''
    # First pass - criteria file defined, and exists
    if criteria is not None:
        if os.path.exists(criteria):
            criteria = self.validate_exists(criteria)
            
    # Second pass, use default provided by library
    if criteria is None:
         criteria = self.default_criteria

    # Attempt to load (and validate) the criteria (returns YamlManager)
    self.criteria = validate_loads(criteria)

    # Again fall back to default if error loading user-provided
    if self.criteria is None:
        self.criteria = validate_loads(default_criteria)

    basename = os.path.basename(self.criteria.infile)

    bot.info('[criteria:%s]' % basename)
    return self.criteria.load()


def validate_criteria(self, criteria=None, infile=None, params=None):
    '''validate an infile (or already loaded one) against criteria.

       Parameters
       ==========
       infile: an input specification file
       criteria: a loaded (json/dict) or criteria, or html/yml file
    '''   
    from openbases.utils import load_module

    # If criteria not set, use default
    if criteria is None:
        self.load_criteria()

    # Update environment with params for validation
    if params is not None:
        bot.info('Updating custom params with %s' % params)
        self.params.update(params)

    # Update the environment
    update_environment(params=self.params, prefix='OPENBASESENV_')

    # Read in the criteria - any errors will fall back to default
    if not isinstance(criteria, dict):
        criteria = self.load_criteria(criteria)

    # Case 1: input file provided at runtime
    if infile is not None:
        self.load(infile)
 
    # Case 2: Didn't successfully load, fall back to previously loaded
    if not hasattr(self, 'spec'): # infile didn't load successfully
        if hasattr(self, 'infile'):
            self.load(infile)

    # Case 3: Still no infile! Exit
    if not hasattr(self, 'spec'):
        bot.error('Please provide an infile to function, or load()')

    if "checks" not in criteria:
        bot.error('criteria is missing "checks" section, exiting.')

    # Default missing function
    missing_function = 'openbases.main.validate.criteria.base.missing'

    # Turn status into meaningful message for user
    lookup = {True: 'pass', False: 'fail', None: 'null'}         

    # Loop through checks, run against specification file
    for group, checks in criteria['checks'].items():

        print('[group|start:%s]' % group)
        values = dict()
        [values.update(dict(check)) for check in checks]
            
        # Obtain values, the only required is the function
        name = values.get('name', self.robot.generate())  
        level = values.get('level', 'warning').upper()
        function = values.get('function', missing_function)
        kwargs = values.get('kwargs')
        envars_list = values.get('environment')
        envars = dict()

        # If we have environment vars from criteria, export them
        if envars_list is not None:
            [update_environment(e) for e in envars_list]

        # If we have a function provided in the configuration yaml
        function_name = function
        function = load_module(function)

        if kwargs is None:
            valid = function(self.spec)
        else:
            values = dict()
            [values.update(dict(kwarg)) for kwarg in kwargs]
            valid = function(self.spec, **values)

        print('[check:%s]' % name)
        print(' test:function %s' % function_name)
        print(' test:result %s' % lookup[valid])

        if valid:
            bot.test("PASSED %s" % function_name)
        else:
            print(' test:level %s' % level)
            bot.named(level, function_name)
            if not valid:
                sys.exit(1)

        print('[group|end:%s]' % group)

    print('ALL TESTS PASSING')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `openbases` package."""

import unittest
import os
import sys

from openbases.main.icons import get_icons

class TestOpenBasesIcons(unittest.TestCase):
    """Tests for `openbases` package."""

    def setUp(self):
        """Setup test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_single(self):
        """Test that default returns single icon"""
        icons = get_icons()
        assert (len(icons) == 1)

    def test_get_multiple(self):
        """Test that N returns more than one"""
        icons = get_icons(N=2)
        assert (len(icons) == 2)

    def test_regexp(self):
        """Test that we can filter based on string"""
        icons = get_icons(regexp="fish")
        for icon in icons:
            assert ("fish" in icon)




# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

# Badges (and color lookup)
# The color needs to follow a # that is converted to %23

CUSTOM_COLORS = {"submission":"green",
                 "spec": "%23660000", # matches schema.org
                 "experiment": "%23eaab1b",
                 "builder": "orange",
                 "openbases": "%232196f3",
                 "testing": "%23783589",
                 "data": "red",
                 "library": "%23ff69b4",
                 "resource": "blue",
                 "paper": "%231ab170",
                 "other": "lightgrey" }

BADGE_STYLES = ["plastic",
                "flat",
                "flat-square",
                "for-the-badge",
                "popout",
                "popout-square",
                "social"]


BADGE_LABELS = list(CUSTOM_COLORS.keys())

# https://img.shields.io/badge/<SUBJECT>-<STATUS>-<COLOR>.sv

BADGE_BASE = "https://img.shields.io/badge/%s-%s-%s.svg"
BADGE_COLORS = ['black',
                'dimgray',
                'dimgrey',
                'gray',
                'grey',
                'darkgray',
                'darkgrey',
                'silver',
                'lightgray',
                'lightgrey',
                'gainsboro',
                'whitesmoke',
                'white',
                'snow',
                'rosybrown',
                'lightcoral',
                'indianred',
                'brown',
                'firebrick',
                'maroon',
                'darkred',
                'red',
                'mistyrose',
                'salmon',
                'tomato',
                'darksalmon',
                'coral',
                'orangered',
                'lightsalmon',
                'sienna',
                'seashell',
                'chocolate',
                'saddlebrown',
                'sandybrown',
                'peachpuff',
                'peru',
                'linen',
                'bisque',
                'darkorange',
                'burlywood',
                'antiquewhite',
                'tan',
                'navajowhite',
                'blanchedalmond',
                'papayawhip',
                'moccasin',
                'orange',
                'wheat',
                'oldlace',
                'floralwhite',
                'darkgoldenrod',
                'goldenrod',
                'cornsilk',
                'gold',
                'lemonchiffon',
                'khaki',
                'palegoldenrod',
                'darkkhaki',
                'ivory',
                'beige',
                'lightyellow',
                'lightgoldenrodyellow',
                'olive',
                'yellow',
                'olivedrab',
                'yellowgreen',
                'darkolivegreen',
                'greenyellow',
                'chartreuse',
                'lawngreen',
                'honeydew',
                'darkseagreen',
                'palegreen',
                'lightgreen',
                'forestgreen',
                'limegreen',
                'darkgreen',
                'green',
                'lime',
                'seagreen',
                'mediumseagreen',
                'springgreen',
                'mintcream',
                'mediumspringgreen',
                'mediumaquamarine',
                'aquamarine',
                'turquoise',
                'lightseagreen',
                'mediumturquoise',
                'azure',
                'lightcyan',
                'paleturquoise',
                'darkslategray',
                'darkslategrey',
                'teal',
                'darkcyan',
                'aqua',
                'cyan',
                'darkturquoise',
                'cadetblue',
                'powderblue',
                'lightblue',
                'deepskyblue',
                'skyblue',
                'lightskyblue',
                'steelblue',
                'aliceblue',
                'dodgerblue',
                'lightslategray',
                'lightslategrey',
                'slategray',
                'slategrey',
                'lightsteelblue',
                'cornflowerblue',
                'royalblue',
                'ghostwhite',
                'lavender',
                'midnightblue',
                'navy',
                'darkblue',
                'mediumblue',
                'blue',
                'slateblue',
                'darkslateblue',
                'mediumslateblue',
                'mediumpurple',
                'rebeccapurple',
                'blueviolet',
                'indigo',
                'darkorchid',
                'darkviolet',
                'mediumorchid',
                'thistle',
                'plum',
                'violet',
                'purple',
                'darkmagenta',
                'fuchsia',
                'magenta',
                'orchid',
                'mediumvioletred',
                'deeppink',
                'hotpink',
                'lavenderblush',
                'palevioletred',
                'crimson',
                'pink',
                'lightpink']



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
import requests
import random
import os
import re
import sys


def get_icons(url="https://openbases.github.io/openbases-icons/icons.json",
              regexp=None, 
              N=1):
    '''get a number of icons from the openbases-icons repository, 
       and return a list.
       Parameters
       ==========
       url: the openbases-icons json list (or other) of icons to choose from       
       regexp: a regular expression to filter by (default None, no filter)
       N: the number of icons to return (default is 1)
       Returns
       =======
       icons: a list of filtered (or not) icons      
    '''
    try:
        icons = requests.get(url).json()
    except:
        bot.error("Error parsing json at %s" % url)

    # Does the user want to filter?
    if regexp is not None:
        icons = [i for i in icons if re.search(regexp, i)]

    # The number of results to return
    random.shuffle(icons)
    
    # A search might not have results
    if len(icons) >= N:
        icons = icons[0:0+N]
    return icons



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import requests
import argparse
import sys
import os

def get_parser():

    from openbases.main.defaults import BADGE_STYLES, BADGE_COLORS

    parser = argparse.ArgumentParser(description="OpenBases Python Badges",
                                formatter_class=argparse.RawTextHelpFormatter,
                                add_help=False)

    # Global Options
    parser.add_argument('--version', dest="version", 
                        help="show openbases python version", 
                        default=False, action='store_true')


    subparsers = parser.add_subparsers(help='description',
                                       title='actions',
                                       description='actions for openbases',
                                       dest="command", metavar='general usage')

    view = subparsers.add_parser("view",
                                 help="view options for style, labels, etc.")

    view.add_argument('choice', nargs="?",
                      default="labels",
                      choices=['labels', 'styles', 'colors'])

    create = subparsers.add_parser("create",
                                    help="extract values from a paper.md")

    create.add_argument('--cache', dest="longCache", 
                        help="enable long cache for badge. Default False", 
                        default=False, action='store_true')

    create.add_argument('names', nargs="*", 
                        help="label followed by name")

    create.add_argument("--link", dest="link", 
                        default=None, type=str,
                        help='badge link, defaults to openbases.github.io')

    create.add_argument('--style',
                        default="flat",
                        choices=BADGE_STYLES, dest="style")

    create.add_argument('--color',
                        default=None,
                        choices=BADGE_COLORS, dest="color")

    create.add_argument('--fmt',
                        default="markdown",
                        choices=["markdown", "svg"], dest="format")

    return parser


def get_subparsers(parser):
    '''get_subparser will get a dictionary of subparsers, to help with printing help
    '''

    actions = [action for action in parser._actions 
               if isinstance(action, argparse._SubParsersAction)]

    subparsers = dict()
    for action in actions:
        # get all subparsers and print help
        for choice, subparser in action.choices.items():
            subparsers[choice] = subparser

    return subparsers


def version():
    '''version prints the version, both for the user and help output
    '''
    import openbases
    return openbases.__version__
    

def main(main=None):

    parser = get_parser()
    subparsers = get_subparsers(parser)

    def help(return_code=0):
        '''print help, including the software version and exit
        '''
        v = version()
        print("\nOpen Bases Badges Python [v%s]\n" %(v))
        parser.print_help()
        sys.exit(return_code)
    
    try:
        # We capture all primary arguments, and take secondary to pass on
        args, options = parser.parse_known_args()
    except:
        sys.exit(0)

    # If the user wants the version
    if args.version is True:
        print(version())
        sys.exit(0)

    # Does the user want help for a subcommand?
    if args.command == 'create': from .create import main 
    elif args.command == 'view': from .view import main 
    else: help()

    # Pass on to the correct parser
    if args.command is not None:
        main(args=args, options=options)


if __name__ == '__main__':
    main()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

def get_client(quiet=False, debug=False):
    '''return the helper function client for OpenBases.
       This is a client that spans the different modules (papers, badges,
       etc.) for interaction with functions from one interface.

       Parameters
       ==========
       quiet: if True, suppress most output about the client
       debug: turn on debugging mode

    '''
    from openbases.main.base import Client
    from openbases.main.validate import ( PaperValidator, BasicValidator )

    Client.quiet = quiet
    Client.debug = debug

    # Paper
    from openbases.main.papers import Paper
    Client.paper = Paper

    # Validation
    Client.BasicValidator = BasicValidator
    Client.PaperValidator = PaperValidator

    # Badge
    from openbases.main.badges import Badge
    Client.badge = Badge

    # Initialize
    cli = Client()
    return cli

Client = get_client()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import os
import re

import json
from openbases.logger import bot
import subprocess
import sys

################################################################################
## REPO OPERATIONS #############################################################
################################################################################


def clone(url, tmpdir=None, branch='master'):
    '''clone a repository from Github'''
    if tmpdir is None:
        tmpdir = tempfile.mkdtemp()
    name = os.path.basename(url).replace('.git', '')
    dest = '%s/%s-%s' %(tmpdir, name, branch)
    return_code = os.system('git clone -b %s %s %s' %(branch, url, dest))
    if return_code == 0:
        return dest
    bot.error('Error cloning repo.')
    sys.exit(return_code)


def get_post_fields(request):
    '''parse through a request, and return fields from post in a dictionary
    '''
    fields = dict()
    for field,value in request.form.items():
        fields[field] = value
    return fields



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# This code was originally written *by same author* for openschemas and shared
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
from openbases.main.validate.helpers import (
    validate_loads,
    validate_criteria,
    load_criteria
)

import os
import sys

here = os.path.abspath(os.path.dirname(__file__))


################################################################################
# Validator Classes ############################################################
################################################################################

class BasicValidator:
    '''the basic validator doesn't care about the input file extensions,
       and gives the user more freedom to write custom validators using it. 
       We only validate that the file exists.
    '''
    default_criteria = '%s/criteria/dummy.yml' % here
    params = {}

    def __str__(self):
        return '[basic-validator:%s]' % self.name

    def __repr__(self):
        return self.__str__()

    def __init__(self, infile):

        from openbases.main.base import RobotNamer
        self.load(infile)
        self.robot = RobotNamer()

# Loading

    def load(self, infile):
        '''load an input file, meaning checking for the file's existence.
           It's up to the user (if desired) to load the file in their 
           validation functions, or write a custom _load() function.

           Parameters
           ==========
           infile: the input file to load
        '''
        # Step 0. of validation checks exist and load of file
        self.infile = self.validate_exists(infile)
        if self.infile is not None:

            # If the fie exists, we can store metadata about it
            self.name = os.path.basename(self.infile)
            self.folder = os.path.dirname(self.infile)
            print('Input file provided under attribute "infile"')


# Validation

    def validate_exists(self, infile):
        '''determine filename of infile, validate that exists. 

           Parameters
           ==========
           infile: the name of the specification / yaml file
           extensions: a list of valid extensions
        '''
        if os.path.exists(infile):
            print('Found %s, valid name' % infile)
            return os.path.abspath(infile)

    
class PaperValidator:
    '''the spec validator can "sniff" a file based on extension, and validate
       the file based on the extension, or have one of the specific type 
       validators (html, yaml) called directly. There are two steps.

       Step 0. validates the file itself. Does it exist? Load without error?
       Step 1. validates the file against a criteria.yml, default is provided
    '''

    default_criteria = '%s/criteria/paper.yml' % here
    params = {}
    folder = None
    name = None

    def __str__(self):
        return '[paper-validator:%s]' % self.name

    def __repr__(self):
        return self.__str__()

    def __init__(self, infile, bibfile=None):
        from openbases.main.base import RobotNamer
        self.load(infile)
        self.robot = RobotNamer()

        # If there is an error or file not found, is None
        self.bib = self.load_bib(bibfile)
        
# Loading

    def load(self, infile):
        '''load an input file, meaning checking for the file's existence,
           that it has a default extension, and loading it into the "spec"
           via the YamlManager (that can handle frontmatter in a html or
           standard yml/yaml

           Parameters
           ==========
           infile: the input file to load
        '''
        # Step 0. of validation checks exist and load of file
        self.infile = self.validate_exists(infile)
        if self.infile is not None:

            # If the fie exists, we can store metadata about it
            self.name = os.path.basename(self.infile)
            self.folder = os.path.dirname(self.infile)

            # We only return a manager if the file loads cleanly.
            self.spec =  validate_loads(infile) # returns YamlManager
            if self.spec is not None:
                return self.spec.load()

    def load_bib(self, bibfile, envar="OPENBASESENV_BIBFILE"):
        '''load the bibfile, and derive from the paper file if it was loaded
           first. This means checking for the file's existence,
           that it has a default extension, and loading it into the "spec"
           
           Parameters
           ==========
           infile: the input file to load
        '''
        # If the bibfile is None, then we try deriving based on paper.md
        if bibfile is None:
            if self.folder is not None:
                bibfile = os.path.join(self.folder, 'paper.bib')
    
        self.bibfile = self.validate_exists(bibfile, extensions=['bib'])
        if self.bibfile is not None:

            # If we have a bibfile, export to environment for tests
            os.putenv(envar, self.bibfile)
            os.environ[envar] = self.bibfile
            self.params.update({'bibfile': self.bibfile})

            # We show error to user, but don't exit here
            from openbases.utils import read_bibtex
            return read_bibtex(self.bibfile)

# Validation

    def validate_exists(self, infile, extensions=None):
        '''determine filename of infile 
           based on Specification Name (and extension). If the extension
           doesn't end in yml/yaml or html, it's not valid (and note
           we will need to add support for reading json). If valid,
           return the filename. If not, return None.

           Parameters
           ==========
           infile: the name of the specification / yaml file
           extensions: a list of valid extensions
        '''
        if extensions == None:
            extensions = ['yaml', 'yml', 'html', 'md', 'markdown']

        if not isinstance(extensions, (list,tuple)):
            extensions = [extensions]

        for ext in extensions:
            if infile.endswith(ext) and os.path.exists(infile):
                print('Found %s, valid name' % infile)
                return os.path.abspath(infile)

        # Tell the user doesn't have valid, show which are
        valids = ','.join(extensions)
        print('%s does not have a valid extension (%s)' % (infile, valids))


BasicValidator.load_criteria = load_criteria
BasicValidator.validate_criteria = validate_criteria
PaperValidator.load_criteria = load_criteria
PaperValidator.validate_criteria = validate_criteria



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `openbases` package."""

import unittest
import os
import re
import sys

from openbases.main.badges import Badge
from openbases.main.defaults import (
    BADGE_STYLES,
    BADGE_LABELS
)

class TestOpenBasesBadges(unittest.TestCase):
    """Tests for `openbases` package."""

    def setUp(self):
        """Setup test fixtures, if any."""
        self.labels = BADGE_LABELS
        self.styles = BADGE_STYLES
        self.badge = Badge(label="experiment", name="labjs")

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_generate_default(self):
        """Test that default is an svg within markdown"""
        baseurl = "https://img.shields.io/badge/experiment-labjs-%23eaab1b.svg"
        assert (self.badge.baseurl == baseurl)

    def test_gets(self):
        """Test that N returns more than one"""
        assert (self.badge.get_link() == 'https://openbases.github.io')
        assert (self.badge.get_style() == 'flat')

    def test_sets(self):
        """Test that N returns more than one"""
        self.badge.set_link('https://www.google.com')
        assert (self.badge.get_link() == 'https://www.google.com')
        self.badge.set_style('flat-square')
        assert (self.badge.get_style() == 'flat-square')
        # Invalid style shouldn't be changed
        self.badge.set_style('hello-kitty')
        assert (self.badge.get_style() == 'flat-square')

    def test_svg(self):
        """ensure that the get_svg() returns an svg of the badge"""
        svg = self.badge.get_svg()
        assert ( 'svg xmlns="http://www.w3.org/2000/svg' in svg)

    def test_markdown(self):
        """ensure that markdown is generated for get_markdown()"""
        markdown = self.badge.get_markdown()
        assert ( re.search("^!\[.+\]\(.+\)$", markdown) is not None )



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
from openbases.utils import (
    read_file, 
    read_frontmatter,
    read_markdown,
    read_yaml,
    write_yaml
)
import os
import sys
import re

class YamlManager:
    '''a YamlManager will read in markdown with Yaml, or pure yaml, and 
       also save the "rest" of the content, if applicable. We have
       removed the python-frontmatter dependencies along with ruamel in 
       favor of just using pyaml for a cleaner installation.
    '''
    infile = None

    def __init__(self, infile=None):
        ''' on init, if a path is provided we want to tell the user quickly
            if it doesn't exist. If the path exists, we also load.

            Parameters
            ==========
            infile: can be a yml file, a markdown file, or html (with frontmatter)
        '''
        self.loaded = {}
        self.content = ''

        # Did the user provide a path to load?
        if infile is not None:
            self.set_infile(infile)

    def set_infile(self, file_path):
        if self._validate_exists(file_path):
            self.infile = file_path
        else:
            bot.warning("%s does not exist." % infile)

    def _validate_exists(self, infile=None):
        '''first determine if the infile is defined, with preference
           to a potentially new file set by the user at runtime. If not set,
           use previously loaded file. In both cases, first check if the
           file exists. Return False if not defined or doesn't exist

           Parameters
           ==========
           infile: a yaml, html, or markdown file
        '''
        if not infile:
            infile = self.infile

        if infile not in ['', None]:
            if os.path.exists(infile):
                return True
            else:
                bot.warning("%s does not exist." % infile)
        else:
            bot.warning("Input (yml/md/html) file is not defined.")
        return False

    def load(self, file_path=None):
        '''load the input file depending on its extension

           Parameters
           ==========
           file_path: a yaml/html file path, if desired, to override previous
        '''

        if not file_path:
            file_path = self.infile

        # Read in raw content
        if self._validate_exists(file_path):

            # Read in standard yaml
            if re.search('[.](yml|yaml)$', file_path):
                self._load_yaml(file_path)

            # Read in html or markdown
            else:
                self._load_frontmatter(file_path)
                self.content = read_markdown(file_path)
            return self.loaded

# Loading

    def _load_yaml(self, file_path, quiet=True):
        '''load the yaml file

           Parameters
           ==========
           file_path: the yaml file path to read
        '''
        self.loaded = read_yaml(file_path, quiet=quiet)

        
    def _load_frontmatter(self, file_path, quiet=True):
        '''load the yaml as frontend matter from an html file

           Parameters
           ==========
           file_path: an html or markdown file path to read
        '''
        self.loaded = read_frontmatter(file_path, quiet=quiet)


# Saving

    def save_yml(self, output_file, content=None, mode = 'w', ext='yml'):
        '''save a yml file, either provided by the client (content)
           or if not provided, the loaded content.
         
           Parameters
           ==========
           output_file: the output file to save to. Should end in yml or yaml
           content: the content to parse to yaml, can be str or dict
           mode: the mode to use (default is w, write)

        '''
        # If content isn't provided, use client loaded content (must be dict)
        if not content:
            content = self.loaded
        
        # Remove any derivation (won'account for compressed e.g., .tar.gz)
        output_file, _ = os.path.splitext(output_file)

        # Ensure ends with a yml derivative extension
        if not re.search('(%s$)' % ext, output_file):
            output_file = "%s.%s" % (output_file, ext)

        # Write the yaml to file
        write_yaml(output_file, content, mode) 
       
# Reading

    def get_key(self, key='specifications'):
        '''return a portion of the yml file based on key

           Parameters
           ==========
           key: defaults to specifications
        '''
        # If not yet loaded, load it based on extension
        if not hasattr(self, 'loaded'):
            self.load(self.infile)
        return self.loaded[key]



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
import os
import re
import requests
import sys

from openbases.main.defaults import (
    BADGE_BASE,
    BADGE_COLORS,
    CUSTOM_COLORS,
    BADGE_LABELS,
    BADGE_STYLES
)

try:
    from urllib import urlencode
except:
    from urllib.parse import urlencode

class Badge:

    def __init__(self, 
                 label,
                 name, 
                 link=None, 
                 color=None,
                 longCache=True,
                 style=None):

        '''create a new badge for an open base. The only required fields are the
           subject and status (e.g., experiment labjs) and the rest have 
           reasonably set defaults

           Parameters
           ==========
           label: the type of badge, if not one in openbases, "other" is used.
           This label corresponds with "subject" used by shields.io
           name: the name for the badge, corresponds with "status" in shields.io
        '''

        self.params = {"style": "flat",
                       "link": "https://openbases.github.io"}

        self._init_design(label, name, color)
        self._init_cache(longCache)
        self.set_style(style)
        self.set_link(link)

    def get_style(self):
        return self.params["style"]

    def get_link(self):
        return self.params["link"]

    def get_color(self):
        return self.color

    def set_style(self, style):
        if style in BADGE_STYLES:
            self.params["style"] = style

    def set_color(self, color):
        if color in BADGE_COLORS:
            self.color = color
            self._update_design() 
        else:
            bot.error("Color 404! Run ob-badge view colors.")
 
    def set_label(self, label):
        '''allow the user to set a custom label (first one on left)'''
        self.label = label
        self._update_design()

    def set_name(self, name):
        '''allow the user to set a custom name (second on right)'''
        self.name = name
        self._update_design()

    def set_link(self, link):
        if link is not None:
            self.params["link"] = link

    def _init_design(self, subject, status, color=None):
        '''the base of the design is the subject, and status.
        
           Parameters
           ==========
           subject: the type of open base, e.g., "experiment"
           status: the name of the open bsae, e.g., "labjs"
        '''
            
        # Look up the color based on the subject
        subject = subject.lower()
        status = status.replace("-","_")

        # Save parameters for later
        self.name = subject
        self.label = status

        if color is None:
            color = CUSTOM_COLORS.get(subject, CUSTOM_COLORS["other"])
        self.color = color   
        self._update_design()

    def _update_design(self):
        '''update design will regenerate the baseurl depending on the color,
           name, and label
        '''      

        # https://img.shields.io/badge/<SUBJECT>-<STATUS>-<COLOR>.sv
        self.baseurl = BADGE_BASE % (self.name, self.label, self.color)
        bot.debug(self.baseurl)
        return self.baseurl

    def _init_cache(self, enable=False):
        '''add a variable to turn on longCache'''
        if enable is True:
            self.params['longCache'] = "true"

    def get_url(self):
        # Add the parameters
        params = urlencode(self.params)
        return "%s?%s" % (self.baseurl, params)

    def get_markdown(self):
        '''return markdown of the badge based on the generated base string
        '''
        url = self.get_url()
        return "![%s](%s)" %(url, url)

    def get_svg(self):
        '''return an svg of the badge by retrieving the url'''
        url = self.get_url()
        return requests.get(url).text

    def __str__(self):
        return "<badge: %s>" % self.baseurl

    def __repr__(self):
        return self.__str__()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

__version__ = "0.0.54"
AUTHOR = 'Vanessa Sochat'
AUTHOR_EMAIL = 'vsochat@stanford.edu'
NAME = 'openbases'
PACKAGE_URL = "http://www.github.com/openbases/openbases-python"
KEYWORDS = 'openbases, markdown, pandoc'
DESCRIPTION = "openbases python helper functions for https://openbases.github.io"
LICENSE = "LICENSE"

INSTALL_REQUIRES = (
    ('pyaml', {'min_version': '17.12.1'}),
    ('requests', {'min_version': None})
)

VALIDATE_REQUIRES = (
    ('pybtex', {'min_version': None}),
)

TEST_REQUIRES = (
    ('pytest', {'min_version': None}),
    ('requests', {'min_version': None})
)




from .fileio import ( 
    mkdir_p, 
    load_module,
    find_files,
    read_bibtex,
    read_file, 
    read_frontmatter,
    read_markdown,
    read_json,
    read_yaml,
    write_file, 
    write_json,
    write_yaml
)

from .web import clone

from .terminal import ( 
    get_installdir,
    stream_command,
    run_command
)



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import sys
from openbases.main import get_client
from openbases.logger import bot

def main(args, options, parser, sep=","):
    
    command = args.cmd
    cli = get_client()
    quiet = not args.debug or args.silent

    
    # Minimum required is <action> paper.md
    if len(command) < 1:
        bot.exit('''Please provide the paper.md file after an action 
                    ob-paper get paper.md)''')

    # If no arguments, show all fields and exit
    if len(command) <=1:
        paper = cli.paper(command[0], quiet=False)

    else:
        paper = cli.paper(command[0], quiet=quiet)

        # For remainder of arguments, get key
        for arg in command[1:]:

            # If the arg is of format arg:field will return field from list
            paper.get(arg, sep=sep)



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

# Validation functions for openbases papers.

# The format of this file is based on levels, messages, and functions.
# It is intended to be run against a single yml file loaded as json. The
# intention is to make it easy to write and understand these criteria. The
# format is the following:

# SECTION DEFINITIONS
# - the version of the criteria format is under version
# - checks: is the group that is run for validation
# - the first field under checks is for a group of checks. You can name it
#   whatever is appropriate for your need (e.g., titles)
# - name: corresponds to the name of the subsection (a check)
# - function: is the python module --> function that is run
# - level: is the python logger level to emit.
# - the subsuections underneath are also different named (individual) checks
# - args: should be a list of named (or not named) arguments.

# It's helpful to look at the example below and then reference the above!

# WRITING FUNCTIONS

# Each function should have the first variable be the file being checked, or
# the loaded specification as a json-derivative (python dictionary). If 
# running from the command line, you would need to provide the path to this
# criteria and the file. If you want to save the redundant loads, you can run
# the validations from within python (and provide the data structure as 
# variable.

# Each of the checks (the functions that you write and provide) should return
# True (passed) or False (failed). You can print output to help the user to
# your heart's content.

from openbases.logger import bot
from openbases.utils import ( find_files, read_file )
import re
import os

def test_length(spec, **kwargs):
    '''the paper should be betwee 250 and 1000 characters.

        length:
          - name: Check that the paper is between 250-1000 characters
          - level: warning
          - function: openbases.main.papers.tests.test_length
    ''' 
    paper = spec.content

    min_length = int(kwargs.get('min_length', 250))
    max_length = int(kwargs.get('max_length', 1000))

    # This currently includes all markdown, additional text, etc.
    bot.warning('Length of paper is approximately %s chars.' % len(paper))
    if len(paper) < min_length:
        bot.warning("Paper (including markdown) is < %s chars" % min_length)

    elif len(paper) > max_length:
        bot.warning("Paper estimated > %s characters" % max_length)

    return True


def test_authors(spec, **kwargs):
    '''the paper header should have a list of authors, each with affiliation,
       name, and orcid.

        authors:
          - name: Submission must include a list of authors, well formatted
          - level: error
          - function: openbases.main.papers.tests.test_authors
    ''' 
    header = spec.loaded

    if "authors" not in header:
        bot.exit('authors missing in front end paper matter.')    
    if not isinstance(header['authors'], list):
        bot.exit('authors in front end paper matter should be a list') 

    # Regular expression for orcid
    regexp = "[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{4}"

                       # key           type has_regexp  regexp 
    required_fields = ('affiliation', 'orcid', 'name')
    for field in required_fields:
        for author in header['authors']:
            if field not in author:
                bot.exit('Paper is missing a required author field %s' % field)
                print(author)
            if field == 'orcid':
                if not re.search(regexp, author[field]):
                    bot.error('Orcid regular expression failed %s' % author[field])
                    print(author)
    return True


def test_license(spec, envar='OPENBASESENV_REPO_BASE', **kwargs):
    '''required_structure looks for a schema's required fields, and issues
       an exit if doesn't exist. To implement this in a criteria.yml:

        license:
          - name: An OSI license is included in the repository
          - level: error
          - function: openbases.main.papers.tests.test_license
          - environment:
             - OPENBASESENV_REPO_BASE: null

    Templates taken from https://github.com/licenses/license-templates
    '''
    # By default, not found.
    found = False

    # A list of terms to indicate open source licenses
    LICENSE_FLAGS = ["GNU Affero General Public License",
                     "MIT License",
                     "Apache",
                     "Creative Commons",
                     "CC0", # Creative Commans 1.0
                     "CC1",
                     "General Public License",
                     "The GNU General Public License",
                     "GPL-2.0",
                     "Free Software Foundation",
                     "GNU LESSER GENERAL PUBLIC",
                     "GPL-3.0",
                     "Mozilla Public License",
                     "free and unencumbered software released into the public",
                     "www.gnu.org",
                     "This program is free software",
                     "www.wtfpl.net",
                     "COMMON DEVELOPMENT AND DISTRIBUTION",  # CCDL
                     '"AS IS"', # bsd 22
                     'as-is'] # zlib

    repo_base = os.environ.get(envar)
    if repo_base is not None:
        if os.path.exists(repo_base):
            licenses = find_files(repo_base, 'LICENSE')
            for license in licenses:
                content = read_file(license, readlines=False)
                matches = [x for x in LICENSE_FLAGS 
                           if re.search(x.lower(), content.lower())]
                if len(matches) > 0:
                    message = 'Found Open Source license flags! %s'
                    bot.info(message %','.join(matches))
                    print(content)
                    found = True
        else:
            bot.error('%s does not exist! Skipping test.' % repo_base)

    else:
        bot.error('''Did not find %s export for repository base! 
                     Skipping license check''' %envar)

    return found



def test_contributing(spec, envar='OPENBASESENV_REPO_BASE', **kwargs):
    '''look for CONTRIBUTING.md or similar


        contributing:
          - name: Software has CONTRIBUTING.md
          - level: error
          - function: openbases.main.papers.tests.test_contributing
          - environment:
             - OPENBASESENV_REPO_BASE: null

    Templates taken from https://github.com/licenses/license-templates
    '''
    # By default, not found.
    found = False

    repo_base = os.environ.get(envar)
    if repo_base is not None:
        if os.path.exists(repo_base):
            contenders = find_files(repo_base, 'LICENSE|license|License')
            if len(contenders) > 0:
                bot.info('Found CONTRIBUTING file(s) in repository!')
                found = True
    return found


def test_references(spec, envar="OPENBASESENV_BIBFILE", **kwargs):
    '''required_structure looks for a schema's required fields, and issues
       an exit if doesn't exist. To implement this in a criteria.yml:
 
        references:
          - name: References are included and complete
          - level: warning
          - function: openbases.main.papers.tests.test_references

    Templates taken from https://github.com/licenses/license-templates
    '''
    from openbases.utils import read_bibtex

    complete = True
    bibfile = os.environ.get(envar)

    # We must have a bibfile
    if not "bibfile" and "bibfile" not in kwargs:
        bot.warning('Bibfile not provided as argument, skipping test')
        return complete

    required_fields = ['title', 'month', 'year']

    bib = read_bibtex(bibfile)
    for entryid, entry in bib.items():
        bot.info('Testing bibliography entry %s' % entryid)
        bot.info('  type: %s' % entry.type)
        for field in entry.fields:
            if field not in entry.fields:
                bot.error('  %s is missing field %s' % (entryid, field))
                complete = False
            if entry.fields[field] in [None, '']:
                bot.error('  %s has empty field %s' % (entryid, field))
                complete = False
        bot.info('  title: %s' % entry.fields['title'])

    return complete



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.main.defaults import (
    BADGE_BASE,
    BADGE_COLORS,
    BADGE_LABELS,
    BADGE_STYLES
)

def main(args, options):

    if args.choice.lower() == "colors":
        print('\n'.join(BADGE_COLORS))    
    if args.choice.lower() == "labels":
        print(' '.join(BADGE_LABELS))
    elif args.choice.lower() == "styles":
        print(' '.join(BADGE_STYLES))



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import sys
from openbases.main.badges import Badge
from openbases.logger import bot

def main(args, options):
    
    if len(args.names) < 2:
        bot.error('Must provide a label and name, e.g., experiment labjs')

    badge = Badge(label=args.names[0], 
                  name=args.names[1],
                  longCache=args.longCache,
                  link=args.link,
                  style=args.style)

    # If the user provided a custom color, change it.
    if args.color is not None:
        badge.set_color(args.color)

    # Return the desired format to the user
    if args.format == "svg":
        print(badge.get_svg())
    else:
        print(badge.get_markdown())



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import requests
import argparse
import sys
import os

def get_parser():

    parser = argparse.ArgumentParser(description="OpenBases Python Icons",
                                formatter_class=argparse.RawTextHelpFormatter,
                                add_help=False)

    # Global Options
    parser.add_argument('--version', dest="version", 
                        help="show openbases python version", 
                        default=False, action='store_true')

    parser.add_argument("--regexp", dest="regexp", default=None,
                        help="regular expression filter for icon name", 
                        type=str)

    parser.add_argument('--help', dest="help", 
                        help="show openbases icons help", 
                        default=False, action='store_true')

    parser.add_argument("--url", dest="url", 
                        default="https://openbases.github.io/openbases-icons/icons.json",
                        help="complete url for json list of icons",
                        type=str)

    parser.add_argument("--n",'--N', dest="N", default=1,
                        help="number of icons to return", 
                        type=int)

    parser.add_argument("--sep", dest="sep", default='\n',
                        help="separator to print icons to screen (default newline)", 
                        type=str)

    return parser


    # Import logger to set
    from openbases.logger import bot
    bot.debug('Logging level %s' %level)
    import openbases

    bot.debug("OpenBases Python Version: %s" % openbases.__version__)

def version():
    '''version prints the version, both for the user and help output
    '''
    import openbases
    return openbases.__version__
    

def main(main=None):

    parser = get_parser()

    def help(return_code=0):
        '''print help, including the software version and exit
        '''
        v = version()
        print("\nOpen Bases Icons [v%s]\n" %(v))
        parser.print_help()
        sys.exit(return_code)
    
    try:
        # We capture all primary arguments, and take secondary to pass on
        args, options = parser.parse_known_args()
    except:
        sys.exit(0)

    # Does the user just want help?
    if args.help:
        help()
        sys.exit(0)

    # If the user wants the version
    if args.version is True:
        print(version())
        sys.exit(0)

    # Get the icons with provided function
    from openbases.main.icons import get_icons
    icons = get_icons(url=args.url, N=args.N, regexp=args.regexp)
    print(args.sep.join(icons))

if __name__ == '__main__':
    main()
